c----------------------------------------------------------
      program ccpol8s_dimer
      implicit real*8 (a-h,o-z)
      PARAMETER (nsite=25,maxdat=3000,maxlin=2000)
      PARAMETER (maxpar=1000)
      parameter (nsites=2*nsite*maxdat)
      common /allpar/ params(maxpar),nparsall

      dimension c(maxlin)
      dimension ElA(3),ElB(3)
      dimension RA(3),RB(3)
      dimension T(3,3),vec(3)
      dimension sites(3,nsite)     ! site coordinates for standard molecule
      dimension sitesAB(3,nsites) ! site coords for each of translated and rotated 2*ndat molecules
      dimension aj(maxlin)

      data zero /0.d0/
      data bohr2a /0.529177249d0/
      data h2kcal /627.510d0/
c
c     cartesian coordinates of the sites (in bohr)
c                                                                  exp  elst  indi
      data sites /
     > 0.00000000000000d+00, 0.0000000d0, 0.1255334885d0,   ! O     x    x     x
     >-1.45365196228170d+00, 0.0000000d0,-0.9961538357d0,   ! H     x    x     x
     > 1.45365196228170d+00, 0.0000000d0,-0.9961538357d0,   ! H     x    x     x
     > 0.00000000000000d+00,-0.2067213d0,-0.2462589474d0,   ! D1         x
     > 0.00000000000000d+00, 0.2067213d0,-0.2462589474d0,   ! D1         x
     > 0.90d0,  0.053d0,  0.64d0,     ! N1
     >-0.90d0,  0.053d0,  0.64d0,     ! N1
     > 0.90d0, -0.053d0,  0.64d0,     ! N1
     >-0.90d0, -0.053d0,  0.64d0,     ! N1
     > 1.29d0,  0.14d0,  -0.91d0,     ! N2
     >-1.29d0,  0.14d0,  -0.91d0,     ! N2
     > 1.29d0, -0.14d0,  -0.91d0,     ! N2
     >-1.29d0, -0.14d0,  -0.91d0,     ! N2
     > 0.56d0,  0.24d0,  -0.48d0,     ! N3
     >-0.56d0,  0.24d0,  -0.48d0,     ! N3
     > 0.56d0, -0.24d0,  -0.48d0,     ! N3
     >-0.56d0, -0.24d0,  -0.48d0,     ! N3
     > 0.91d0,  0.21d0,  -0.68d0,     ! N4
     >-0.91d0,  0.21d0,  -0.68d0,     ! N4
     > 0.91d0, -0.21d0,  -0.68d0,     ! N4
     >-0.91d0, -0.21d0,  -0.68d0,     ! N4
     > 1.48d0,  0.26d0,  -0.62d0,     ! N5
     >-1.48d0,  0.26d0,  -0.62d0,     ! N5
     > 1.48d0, -0.26d0,  -0.62d0,     ! N5
     >-1.48d0, -0.26d0,  -0.62d0/     ! N5

      pi180=dacos(-1.d0)/180.d0

      open (7,file='data')
      read (7,*) nparsall
      if (nparsall.gt.maxpar) stop 010
      do 10 i=1,nparsall
         read (7,*) ii,params(i)
         if (ii.ne.i) stop 020
 10   continue

      read (7,*) nlin0
      if (nlin0.gt.maxlin) stop 030
      do 20 i=1,nlin0
         read (7,*) ii,c(i)
         if (ii.ne.i) stop 040
 20   continue
      close(7)

      write (*,'(a)') 'dimer energies (hartree,kcal/mol)'
      read (*,*) ndat
      if (ndat.gt.maxdat) stop 111

      do 200 nd=1,ndat
      read (*,*) dist, ElA(2),ElA(3), ElB(1),ElB(2),ElB(3)   ! bohr,degrees

c switch to radians
      ElA(1)=zero
      do 5 i=1,3
      ElA(i)=ElA(i)*pi180
      ElB(i)=ElB(i)*pi180
 5    continue

      RA(1)=zero
      RA(2)=zero
      RA(3)=zero
      RB(1)=zero
      RB(2)=zero
      RB(3)=dist

c calculate site positions (in bohr) after rotations and translations
      call rotmatA (ElA(1),ElA(2),ElA(3), T)
      indA = (nd-1)*nsite*2 
      indB = (nd-1)*nsite*2 +nsite
      do 201 ns=1,nsite
         call Av (T,sites(1,ns), vec)
         sitesAB(1,indA+ns) =vec(1)+RA(1)
         sitesAB(2,indA+ns) =vec(2)+RA(2)
         sitesAB(3,indA+ns) =vec(3)+RA(3)
 201  continue
      call rotmatA (ElB(1),ElB(2),ElB(3), T)
      do 202 ns=1,nsite
         call Av (T,sites(1,ns), vec)
         sitesAB(1,indB+ns) =vec(1)+RB(1)
         sitesAB(2,indB+ns) =vec(2)+RB(2)
         sitesAB(3,indB+ns) =vec(3)+RB(3)
 202  continue

      call indN_iter (2, sitesAB(1,indA+1), nsite, Eind)

      Eind = Eind           ! hartree
c      Eind = Eind*h2kcal    ! kcal/mol

      indA = (nd-1)*nsite*2 +1
      indB = (nd-1)*nsite*2 +nsite +1
      call U0 
     >(nsite,sitesAB(1,indA),sitesAB(1,indB),dummy, a0,aj,nlin)

      E=Eind
      do 210 nl=1,nlin
      E=E +c(nl)*aj(nl)
 210  continue
      E=E+a0

      write (*,'(i5,2f20.8)') nd,E,E*h2kcal

 200  continue

      end
c----------------------------------------------------------------------
      subroutine U0 (nsite,sitesA,sitesB,energy, a0,aj,nlin)
      implicit real*8 (a-h,o-z)
      PARAMETER (maxpar=1000)

      dimension sitesA(3,*),sitesB(3,*), aj(*)
      common /allpar/ params(maxpar),nparsall

      dimension ind_charge(25)
      dimension ind_beta(25,25)
      dimension ind_c0(25,25)
      dimension ind_c1(25,25)
      dimension ind_c2(25,25)
      dimension ind_c3(25,25)
      dimension ind_d1(25,25)
      dimension ind_d6(25,25)
      dimension ind_d8(25,25)
      dimension ind_d10(25,25)
      dimension ind_C6(25,25)
      dimension ind_C8(25,25)
      dimension ind_C10(25,25)

      data ind_charge /25,26,26,27,27,20*00/

c         O   H   H  D1  D1  N1  N1  N1  N1  N2  N2  N2  N2  N3  N3  N3  N3  N4  N4  N4  N4  N5  N5  N5  N5
      data ind_beta 
     > /034,035,035,102,102,103,103,103,103,104,104,104,104,
     >              105,105,105,105,120,120,120,120,127,127,127,127,  ! O
     >  035,036,036,106,106,107,107,107,107,108,108,108,108,
     >              109,109,109,109,121,121,121,121,128,128,128,128,  ! H
     >  035,036,036,106,106,107,107,107,107,108,108,108,108,
     >              109,109,109,109,121,121,121,121,128,128,128,128,  ! H
     >  102,106,106,110,110,111,111,111,111,112,112,112,112,
     >              113,113,113,113,122,122,122,122,129,129,129,129,  ! D1
     >  102,106,106,110,110,111,111,111,111,112,112,112,112,
     >              113,113,113,113,122,122,122,122,129,129,129,129,  ! D1
     >  103,107,107,111,111,114,114,114,114,115,115,115,115,
     >              116,116,116,116,123,123,123,123,130,130,130,130,  ! N1
     >  103,107,107,111,111,114,114,114,114,115,115,115,115,
     >              116,116,116,116,123,123,123,123,130,130,130,130,  ! N1
     >  103,107,107,111,111,114,114,114,114,115,115,115,115,
     >              116,116,116,116,123,123,123,123,130,130,130,130,  ! N1
     >  103,107,107,111,111,114,114,114,114,115,115,115,115,
     >              116,116,116,116,123,123,123,123,130,130,130,130,  ! N1
     >  104,108,108,112,112,115,115,115,115,117,117,117,117,
     >              118,118,118,118,124,124,124,124,131,131,131,131,  ! N2
     >  104,108,108,112,112,115,115,115,115,117,117,117,117,
     >              118,118,118,118,124,124,124,124,131,131,131,131,  ! N2
     >  104,108,108,112,112,115,115,115,115,117,117,117,117,
     >              118,118,118,118,124,124,124,124,131,131,131,131,  ! N2
     >  104,108,108,112,112,115,115,115,115,117,117,117,117,
     >              118,118,118,118,124,124,124,124,131,131,131,131,  ! N2
     >  105,109,109,113,113,116,116,116,116,118,118,118,118,
     >              119,119,119,119,125,125,125,125,132,132,132,132,  ! N3
     >  105,109,109,113,113,116,116,116,116,118,118,118,118,
     >              119,119,119,119,125,125,125,125,132,132,132,132,  ! N3
     >  105,109,109,113,113,116,116,116,116,118,118,118,118,
     >              119,119,119,119,125,125,125,125,132,132,132,132,  ! N3
     >  105,109,109,113,113,116,116,116,116,118,118,118,118,
     >              119,119,119,119,125,125,125,125,132,132,132,132,  ! N3
     >  120,121,121,122,122,123,123,123,123,124,124,124,124,
     >              125,125,125,125,126,126,126,126,133,133,133,133,  ! N4
     >  120,121,121,122,122,123,123,123,123,124,124,124,124,
     >              125,125,125,125,126,126,126,126,133,133,133,133,  ! N4
     >  120,121,121,122,122,123,123,123,123,124,124,124,124,
     >              125,125,125,125,126,126,126,126,133,133,133,133,  ! N4
     >  120,121,121,122,122,123,123,123,123,124,124,124,124,
     >              125,125,125,125,126,126,126,126,133,133,133,133,  ! N4
     >  127,128,128,129,129,130,130,130,130,131,131,131,131,
     >              132,132,132,132,133,133,133,133,134,134,134,134,  ! N5
     >  127,128,128,129,129,130,130,130,130,131,131,131,131,
     >              132,132,132,132,133,133,133,133,134,134,134,134,  ! N5
     >  127,128,128,129,129,130,130,130,130,131,131,131,131,
     >              132,132,132,132,133,133,133,133,134,134,134,134,  ! N5
     >  127,128,128,129,129,130,130,130,130,131,131,131,131,
     >              132,132,132,132,133,133,133,133,134,134,134,134/  ! N5

      data ind_d1 /28,29,29,31,31,20*00,
     >             29,30,30,32,32,20*00,
     >             29,30,30,32,32,20*00,
     >             31,32,32,33,33,20*00,
     >             31,32,32,33,33,20*00,
     >             25*00,
     >             25*00,
     >             25*00,
     >             25*00,
     >             25*00,
     >             25*00,
     >             25*00,
     >             25*00,
     >             25*00,
     >             25*00,
     >             25*00,
     >             25*00,
     >             25*00,
     >             25*00,
     >             25*00,
     >             25*00,
     >             25*00,
     >             25*00,
     >             25*00,
     >             25*00/

      data ind_d6 /93,94,94,22*00,
     >             94,95,95,22*00,
     >             94,95,95,22*00,
     >             25*00,
     >             25*00,
     >             25*00,
     >             25*00,
     >             25*00,
     >             25*00,
     >             25*00,
     >             25*00,
     >             25*00,
     >             25*00,
     >             25*00,
     >             25*00,
     >             25*00,
     >             25*00,
     >             25*00,
     >             25*00,
     >             25*00,
     >             25*00,
     >             25*00,
     >             25*00,
     >             25*00,
     >             25*00/

      data ind_d8 /96,97,97,22*00,
     >             97,98,98,22*00,
     >             97,98,98,22*00,
     >             25*00,
     >             25*00,
     >             25*00,
     >             25*00,
     >             25*00,
     >             25*00,
     >             25*00,
     >             25*00,
     >             25*00,
     >             25*00,
     >             25*00,
     >             25*00,
     >             25*00,
     >             25*00,
     >             25*00,
     >             25*00,
     >             25*00,
     >             25*00,
     >             25*00,
     >             25*00,
     >             25*00,
     >             25*00/

      data ind_d10/99,100,100,22*00,
     >            100,101,101,22*00,
     >            100,101,101,22*00,
     >             25*00,
     >             25*00,
     >             25*00,
     >             25*00,
     >             25*00,
     >             25*00,
     >             25*00,
     >             25*00,
     >             25*00,
     >             25*00,
     >             25*00,
     >             25*00,
     >             25*00,
     >             25*00,
     >             25*00,
     >             25*00,
     >             25*00,
     >             25*00,
     >             25*00,
     >             25*00,
     >             25*00,
     >             25*00/

      data ind_C6 /84,85,85,22*00,
     >             85,86,86,22*00,
     >             85,86,86,22*00,
     >             25*00,
     >             25*00,
     >             25*00,
     >             25*00,
     >             25*00,
     >             25*00,
     >             25*00,
     >             25*00,
     >             25*00,
     >             25*00,
     >             25*00,
     >             25*00,
     >             25*00,
     >             25*00,
     >             25*00,
     >             25*00,
     >             25*00,
     >             25*00,
     >             25*00,
     >             25*00,
     >             25*00,
     >             25*00/

      data ind_C8 /87,88,88,22*00,
     >             88,89,89,22*00,
     >             88,89,89,22*00,
     >             25*00,
     >             25*00,
     >             25*00,
     >             25*00,
     >             25*00,
     >             25*00,
     >             25*00,
     >             25*00,
     >             25*00,
     >             25*00,
     >             25*00,
     >             25*00,
     >             25*00,
     >             25*00,
     >             25*00,
     >             25*00,
     >             25*00,
     >             25*00,
     >             25*00,
     >             25*00,
     >             25*00,
     >             25*00/

      data ind_C10/90,91,91,22*00,
     >             91,92,92,22*00,
     >             91,92,92,22*00,
     >             25*00,
     >             25*00,
     >             25*00,
     >             25*00,
     >             25*00,
     >             25*00,
     >             25*00,
     >             25*00,
     >             25*00,
     >             25*00,
     >             25*00,
     >             25*00,
     >             25*00,
     >             25*00,
     >             25*00,
     >             25*00,
     >             25*00,
     >             25*00,
     >             25*00,
     >             25*00,
     >             25*00,
     >             25*00/

      data zero /0.d0/
      data h2kcal /627.510d0/
      
      nlin=144

      do 5 i=1,nlin
      aj(i)=zero
 5    continue

      E_exp=zero
      E_ele=zero
      E_ind=zero

      do 10 nsA=1,nsite
      do 10 nsB=1,nsite

      call distan (sitesA(1,nsA),sitesB(1,nsB),R)

      if (ind_beta(nsA,nsB).ne.0) then     ! exponential
         beta = params(ind_beta(nsA,nsB))
         eks = dexp(-beta*R)
         indlin=ind_beta(nsA,nsB)-98
         if (indlin.lt.0) indlin=indlin+65
         ind0= indlin
         ind1= ind0+36
         ind2= ind1+36
         ind3= ind2+36
         aj(ind0) = aj(ind0) +eks
         aj(ind1) = aj(ind1) +eks*R
         aj(ind2) = aj(ind2) +eks*R*R
         aj(ind3) = aj(ind3) +eks*R*R*R
c         E_exp=...
      end if

      if (ind_charge(nsA)*ind_charge(nsB).ne.0) then     ! elst
         qA = params(ind_charge(nsA))
         qB = params(ind_charge(nsB))
         d1 = params(ind_d1(nsA,nsB))
         f1 = damp(1,d1,R)
         E_ele = E_ele+ f1* qA*qB/R
      end if

      if (ind_d6(nsA,nsB).ne.0) then     ! ind-disp
         d6 = params(ind_d6(nsA,nsB))
         d8 = params(ind_d8(nsA,nsB))
         d10 = params(ind_d10(nsA,nsB))
         C6 = params(ind_C6(nsA,nsB))
         C8 = params(ind_C8(nsA,nsB))
         C10 = params(ind_C10(nsA,nsB))
         f6 = damp(6,d6,R)
         f8 = damp(8,d8,R)
         f10 = damp(10,d10,R)
         R2 = R*R
         R6 = R2*R2*R2
         R8 = R6*R2
         R10 = R8*R2
         E_ind = E_ind -f6*C6/R6 -f8*C8/R8 -f10*C10/R10
      end if

 10   continue

c      energy = E_exp +E_ele +E_ind
      a0 = E_ele +E_ind

c!test1
c      write (8,*) 'E_exp=',E_exp*h2kcal
c      write (8,*) 'E_ele=',E_ele*h2kcal
c      write (8,*) 'E_ind=',E_ind*h2kcal
c      write (8,*) 'energy=',energy*h2kcal
c      sum=zero
c      do 200 i=1,40
c      sum=sum + aj(i)*params(i+43)
c 200  continue
c      write (8,*) 'test:',sum*h2kcal
c!test0      


      end
c---------------------------------------------------------------------
      subroutine indN_iter(N,sites,nsite,energy)
c Compute the iterative induction between N water molecules.
c The electrostatic field supplied by routine efield.
c
c Assume that the dipole center is at NOT the center
c of mass, but in special point.
      implicit real*8 (a-h,o-z)
      PARAMETER (Nmax=1024)
      PARAMETER (maxsite=30)
      parameter (maxNsite=Nmax*maxsite)
      parameter (maxit=200)

      dimension sites(3,maxNsite) ! site coords for all molecules
      dimension R(3,Nmax)         ! positions of polarizable sites
      dimension G2(3,Nmax)        ! induced dipoles
      dimension E0x(Nmax),E0y(Nmax),E0z(Nmax) ! fields from static charges
      dimension epom(3)
      dimension dist(Nmax,Nmax)   ! distances between pol.centers (**(-3))
      
      data pol /9.922d0/         ! exprmt average polarizability
      data sig /0.367911875040999981d0/ ! position of polarizable center 
                                        ! same as in 3B fits
      data plen /1.1216873242d0/
      data dmpfct /1.d0/
      data zero /0.d0/

      do j=1,N
       do i=1,3
        G2(i,j) = zero
       enddo
      enddo

c Compute positions of polarizable sites and the distance between them
c      dist(i,j)= 1.d0/rij**3

      do 10 i=1,N
      ind1= (i-1)*nsite +1
      ind2= (i-1)*nsite +2
      ind3= (i-1)*nsite +3
      do 11 ii=1,3
       pom = 0.5d0* (sites(ii,ind2) + sites(ii,ind3))
       R(ii,i) = sites(ii,ind1) + sig*(pom-sites(ii,ind1))/plen
 11   continue
 10   continue

      do 20 i=1,N
      do 21 j=i+1,N
      dist(i,j) = 0.d0
      do 25 ii=1,3
 25   dist(i,j) = dist(i,j) +(R(ii,i)-R(ii,j))**2.d0
      dist(i,j)=dist(i,j)**(-1.5d0)
      dist(j,i)=dist(i,j)
 21   continue
 20   continue

c
c Permanent charges fields
c
      do 30 i=1,N
      E0x(i) = zero
      E0y(i) = zero
      E0z(i) = zero
      do 35 j=1,N
      if (j.eq.i) goto 35
      index= (j-1)*nsite +1
      call efield_bohr(R(1,i),sites(1,index),nsite,epom) ! field of j on i
      E0x(i) = E0x(i) +epom(1)
      E0y(i) = E0y(i) +epom(2)
      E0z(i) = E0z(i) +epom(3)
 35   continue
 30   continue


C   LETS DO IT (the iterations, that is...)

      thr_iter = 1.d-20
      change = 10.d0
      isteps = 0

      DO WHILE (change.gt.thr_iter.and.isteps.lt.maxit)   
c     -----------------------------------------------

      energy= 0.0d0
      change= 0.0d0

      do 100 i=1,N
       E1X = E0x(i)
       E1Y = E0y(i) 
       E1Z = E0z(i) 
       do 200 j=1,N
          if(j.eq.i) go to 200
c calculate the total electric field at G1 of molecule i
          call Tprod(R(1,i),R(1,j),G2(1,j),dist(i,j),epom)
          E1X = E1X + dmpfct*epom(1)
          E1Y = E1Y + dmpfct*epom(2)
          E1Z = E1Z + dmpfct*epom(3)
 200   continue

c total field at ii ready. Place induced dipole....
      polE1x = pol*E1X
      polE1y = pol*E1Y
      polE1z = pol*E1Z

      change=(G2(1,i)-polE1X)**2
     >      +(G2(2,i)-polE1Y)**2
     >      +(G2(3,i)-polE1Z)**2 + change
      G2(1,i)=polE1X
      G2(2,i)=polE1Y
      G2(3,i)=polE1Z
      energy =-0.5d0*pol*(E1X*E0X(i)
     >                   +E1Y*E0Y(i)
     >                   +E1Z*E0Z(i)) + energy

 100  continue
      
c      write (*,*) isteps,energy
      isteps = isteps +1
      END DO     !  end while
c     -----------------------

      if (isteps.ge.maxit) then
         write (*,*) 'No convergence in indN_iter'
         write (*,'(a,g12.3)') 'energy change=',change
         write (*,'(a,g12.3)') 'thr_iter=',thr_iter
         stop
      end if

c      energy = 627.510d0*energy   ! hartree--->kcal/mol
      end
c----------------------------------------------------------
c     
c Calculate the contribution of molecule b to the field on a
c Use the CC-pol site charges.
c Units: input positions of bohr, 
c output fields in au
c
      subroutine efield_bohr(veci,sitebt,nsiteb,e)
      implicit real*8 (a-h,o-z)
      PARAMETER (nsitemax=30)

      dimension veci(3),sitebt(3,1),e(3),chrg(25)
      dimension sep(3,nsitemax),sepl(nsitemax)
c      data a0 /0.529177249d0/
      data a0 /1.d0/
      data crgfct /18.22262373d0/
      data chrg /0.292574106d0,0.560656704d0,0.560656704d0,
     >          -0.706943757d0,-0.706943757d0,
     >           0.d0, 0.d0, 0.d0, 0.d0,
     >           0.d0, 0.d0, 0.d0, 0.d0,
     >           0.d0, 0.d0, 0.d0, 0.d0,
     >           0.d0, 0.d0, 0.d0, 0.d0,
     >           0.d0, 0.d0, 0.d0, 0.d0/

c
c Position of polarizable center of A is in veci(*)
      do isite=1,nsiteb
       if(chrg(isite).ne.0.d0) then   ! if charged site
       sepl(isite) = 0.d0
       do k=1,3
        sep(k,isite) = veci(k) - sitebt(k,isite)
        sepl(isite) = sepl(isite) + sep(k,isite)*sep(k,isite)
       end do
       sepl(isite) = sepl(isite)**(-1.5d0)
       endif                            ! if charged site
      end do

      do k=1,3
       e(k) = 0.d0
      end do

      do isite=1,nsiteb
       if(chrg(isite).ne.0.d0) then   ! if charged site
        do k=1,3
         e(k) = e(k) + a0*a0*chrg(isite)*sep(k,isite)*sepl(isite)
        end do
       endif      ! if charged site
      end do
c
      return
      end
c----------------------------------------------------------
c Calculate the vector V resulting from the action
c of the dipole propagator tensor T_ij on U
c Ri, Rj - position vectors of molecules i and j, respectively
c rij = |Ri - Rj|^(-3)
c
        subroutine Tprod(Ri,Rj,u,rij,v)
        implicit real*8 (a-h,o-z)
        dimension Ri(3),Rj(3),u(3),v(3)
c
        ddd = rij**(0.66666666666666666d0)
        scal = 0.d0
        do i=1,3
         v(i) = Ri(i) - Rj(i)
         scal = scal + v(i)*u(i)
        end do
        do i=1,3
         v(i) = (3.d0*v(i)*scal*ddd - u(i))*rij
        end do
c
        return
        end
c----------------------------------------------------------
      subroutine Av (A,v,b)
c calculates A*v=b
      implicit real*8 (a-h,o-z)
      dimension A(3,3),v(3),b(3)
      data zero /0.d0/

      do 10 i=1,3
      sum=zero
      do 20 j=1,3
      sum=sum +a(i,j)*v(j)
 20   continue
      b(i)=sum
 10   continue

      end
c----------------------------------------------------------
      subroutine distan (R1,R2,d)
      implicit real*8 (a-h,o-z)
      dimension R1(3),R2(3)

      d=0.d0
      do 10 i=1,3
      R12i=R1(i)-R2(i)
      d=d +R12i*R12i
 10   continue
      d=sqrt(d)
      end
c----------------------------------------------------------
      function damp (n,beta,r)
c
c     calculate the damping factor (small R correct)
c
      implicit real*8 (a-h,o-z)
      br=beta*r
c The following line added by RB, Sept. 18 1997
      if(br.eq.0.d0) then
       damp = 0.d0
       return
      endif
      sum=1.0d0
      term=1.0d0
      ncn=n
      do i=1,ncn
        term=term*br/i
        sum=sum+term
      enddo
      damp=1.0d0 - dexp(-br)*sum
c     in case of d --> 0 use
c     d=1.0d0 - dexp(-br)*sum = sum_m=ncn+1^\infty br^m/m!
      if(dabs(damp).lt.1.0d-8) then
        damp=0.0d0
        do i=ncn+1,1000
          term=term*br/i
          damp=damp+term
          if(term/damp .lt. 1.0d-8) go to 111
        enddo
        write(6,*) 'No convergence in damp'
  111 continue
      damp=damp*dexp(-br)
      endif
c     write(6,'(i4,2f10.5,e20.10)') n,beta,r,d
      return
      end
c----------------------------------------------------------
        subroutine rotmatA(ad,bd,gd,u)
c
c Construct the rotation matrix corresponding to
c the Euler angles ad,bd,gd (in radians).
c
        implicit real*8 (a-h,o-z)
        dimension u(3,3)
        sad = dsin(ad)
        sbd = dsin(bd)
        sgd = dsin(gd)
        cad = dcos(ad)
        cbd = dcos(bd)
        cgd = dcos(gd)
c----- construct the transformation matrix
        u(1,1) = cad*cbd*cgd - sad*sgd
        u(1,2) = -cad*cbd*sgd - sad*cgd
        u(1,3) = cad*sbd
        u(2,1) = sad*cbd*cgd + cad*sgd
        u(2,2) = -sad*cbd*sgd + cad*cgd
        u(2,3) = sad*sbd
        u(3,1) = -sbd*cgd
        u(3,2) = sbd*sgd
        u(3,3) = cbd
        return
        end
c----------------------------------------------------------
